@@ -1,9 +1,11 @@
 import datetime
+import json
 import os
 import platform
 import sys
 import traceback
-from typing import Optional
+from typing import Any, Optional
+from urllib.parse import urlparse
 
 
 import click
@@ -12,6 +14,8 @@ from pkg_resources import DistributionNotFound, get_distribution
 import requests
 from requests.exceptions import RequestException
 from requests_toolbelt.sessions import BaseUrlSession
+import sentry_sdk
+from sentry_sdk import capture_exception, configure_scope, push_scope
 from xdg import BaseDirectory
 
 __version__ = None
@@ -21,7 +25,8 @@ except DistributionNotFound:
     pass
 
 BOILER_CONFIG_PATH = __name__
-BOILER_CREDENTIAL_FILE = 'credentials'
+BOILER_CONFIG_FILE = 'config'
+SENTRY_DSN = 'https://639591f20e1148c3a65e5e9237c6afc1@o157137.ingest.sentry.io/5224648'
 
 
 class BoilerException(Exception):
@@ -40,9 +45,11 @@ class BoilerSession(BaseUrlSession):
 
     page_size = 50
 
-    def __init__(self, base_url, token):
+    def __init__(self, base_url: str, token: Optional[str]):
         base_url = f'{base_url.rstrip("/")}/'  # tolerate input with or without trailing slash
         super(BoilerSession, self).__init__(base_url=base_url)
+        if token:
+            token = token.strip()
         self.token = token
         self.headers.update(
             {
@@ -59,7 +66,7 @@ class BoilerSession(BaseUrlSession):
             click.echo(
                 click.style(
                     "You are attempting to perform an authorized operation but you aren't logged in.\n"  # noqa
-                    'Run the following command: boiler login',
+                    'Run the following command: boiler login stumpf',
                     fg='yellow',
                 ),
                 err=True,
@@ -88,8 +95,10 @@ def newer_version_available():
 
 def main():
     try:
+        # make boiler configuration directory
+        BaseDirectory.save_config_path(BOILER_CONFIG_PATH)
         cli()
-    except Exception:
+    except Exception as e:
         click.echo(
             click.style(
                 'The following unexpected error occurred while attempting your operation:\n',
@@ -108,7 +117,8 @@ def main():
 
         click.echo(
             click.style(
-                'This is a bug in boiler and should be reported here with the above output:',
+                'This is a bug in boiler and has already been reported. '
+                'If you would like to add any detail you can open an issue below.',
                 fg='yellow',
             ),
             err=True,
@@ -117,14 +127,40 @@ def main():
             'https://gitlab.com/diva-mturk/stumpf-diva/issues/new', err=True,
         )
 
+        with push_scope() as scope:
+            scope.set_extra('boiler_version', f'v{__version__}')
+            scope.set_extra('python_version', f'v{platform.python_version()}')
+            scope.set_extra('os', f'{platform.platform()}')
+            scope.set_extra('command', f'{" ".join(sys.argv[1:])}')
+            capture_exception(e)
 
-def _get_boiler_token() -> Optional[str]:
+
+def update_config_value(filename: str, key: str, value: Any) -> None:
     config_dir = BaseDirectory.load_first_config(BOILER_CONFIG_PATH)
+    config_file = os.path.join(config_dir, filename)
+
+    if os.path.exists(config_file):
+        with open(config_file, 'r') as infile:
+            config = json.load(infile)
+            config[key] = value
+    else:
+        config = {key: value}
+
+    with open(config_file, 'w') as outfile:
+        json.dump(config, outfile, indent=4)
+
+
+def get_config_value(filename: str, key: str) -> Optional[Any]:
+    config_dir: Optional[str] = BaseDirectory.load_first_config(BOILER_CONFIG_PATH)
 
     if config_dir:
-        credential_file = os.path.join(config_dir, BOILER_CREDENTIAL_FILE)
-        if os.path.exists(credential_file):
-            return open(credential_file, 'r').read()
+        config_file = os.path.join(config_dir, BOILER_CONFIG_FILE)
+
+        if os.path.exists(config_file):
+            with open(config_file, 'r') as infile:
+                config = json.load(infile)
+                return config.get(key)
+
     return None
 
 
@@ -132,11 +168,26 @@ def _get_boiler_token() -> Optional[str]:
 @click.option(
     '--api-url', default='https://stumpf.avidannotations.com/api/diva/', envvar='STUMPF_API_URL',
 )
-@click.option('--x-stumpf-token', envvar='X_STUMPF_TOKEN', default=_get_boiler_token)
+@click.option(
+    '--x-stumpf-token',
+    envvar='X_STUMPF_TOKEN',
+    default=get_config_value(BOILER_CONFIG_FILE, 'stumpf_token'),
+)
 @click.option('--offline', is_flag=True)
+@click.option(
+    '--gitlab-url', envvar='GITLAB_URL', default='https://kwgitlab.kitware.com', hidden=True
+)
+@click.option(
+    '--gitlab-project-id', type=click.INT, envvar='GITLAB_PROJECT_ID', default=497, hidden=True
+)
 @click.version_option()
 @click.pass_context
-def cli(ctx, api_url, x_stumpf_token, offline):
+def cli(ctx, api_url, x_stumpf_token, offline, gitlab_url, gitlab_project_id):
+    api = urlparse(api_url)
+
+    if api.netloc == 'stumpf.avidannotations.com':
+        sentry_sdk.init(SENTRY_DSN)
+
     if not offline:
         try:
             if newer_version_available():
@@ -158,11 +209,35 @@ If you are using pipx, then you can upgrade by running the following command:
             )
             raise
 
-    # make boiler configuration directory
-    BaseDirectory.save_config_path(BOILER_CONFIG_PATH)
+    # remove old text config file at 'credentials'
+    # this isn't strictly necessary, it just cleans up an unused file on their machine.
+    # TODO: remove this after a short period
+    config_dir = BaseDirectory.load_first_config(BOILER_CONFIG_PATH)
+    if os.path.exists(os.path.join(config_dir, 'credentials')):
+        os.remove(os.path.join(config_dir, 'credentials'))
 
     session = BoilerSession(api_url, x_stumpf_token)
-    ctx.obj = {'session': session, 'stumpf_url': api_url.replace('/api/diva', '').rstrip('/')}
+    ctx.obj = {
+        'session': session,
+        'stumpf_url': api_url.replace('/api/diva', '').rstrip('/'),
+        'gitlab_url': gitlab_url.rstrip('/'),
+        'gitlab_project_id': gitlab_project_id,
+    }
+
+    # set current user under sentry
+    # ignore any error from requests since the user may be trying to perform a local operation
+    with configure_scope() as scope:
+        try:
+            user = requests.get(
+                f'{ctx.obj["stumpf_url"]}/api/v1/user/me',
+                headers={'X-Stumpf-Token': x_stumpf_token},
+                timeout=(5, 5),
+            )
+
+            if user.ok:
+                scope.user = user.json()
+        except RequestException:
+            pass
 
 
 # TODO: re-enable kpf once deserialization is fixed
@@ -170,6 +245,7 @@ from boiler.commands import (  # noqa: F401 E402
     activity,
     export,
     gunrunner,
+    janitor,
     kw18,
     login,
     mturk,
